# 预训练模型地址

__all__ = ["models_url"]

models_url = {
    "darknet53" : "https://pan.baidu.com/s/1hxiKHXUdS0VE2h_b4N9B3g?pwd=icet",
    "ghostnet"  : "https://pan.baidu.com/s/1Hu70PL0oS5J7_5YwUj6uOw?pwd=icet",
    "vgg11_bn"  : "https://download.pytorch.org/models/vgg11_bn-6002323d.pth",
    "vgg13_bn"  : "https://download.pytorch.org/models/vgg13_bn-abd245e5.pth",
    "vgg16_bn"  : "https://download.pytorch.org/models/vgg16_bn-6c64b313.pth",
    "vgg19_bn"  : "https://download.pytorch.org/models/vgg19_bn-c79401a0.pth",
}